<?php

namespace Drupal\cryptophoto\Controller;

require_once("lib/CryptoPhotoUtils.php");

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

/**
 * CryptoPhoto authentication controller.
 */
class CryptoPhotoAuthController extends ControllerBase {
  public $request;

  public function __construct(RequestStack $request) {
    $this->request = $request;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack')
    );
  }

  /*
   * Programatically log an user
   */
  private function logUser($uid) {
    $user = \Drupal\user\Entity\User::load($uid);
    $user->cpauth = 1;
    user_login_finalize($user);

    global $base_url;
    return new RedirectResponse($base_url);
  }

  /*
   * Handle CryptoPhoto authentication
   */
  public function cp_auth() {
    // disable cache
    \Drupal::service('page_cache_kill_switch')->trigger();

    // get request and config
    $req = $this->request->getCurrentRequest();
    $config = $this->config('cryptophoto.settings');

    // get CryptoPhoto setup
    $privatekey = $config->get('privatekey');
    $publickey = $config->get('publickey');
    $salt = $config->get('salt');
    $server = $config->get('server');

    // error messages + authentication form
    $output = array();
    $output['error_message'] = array();
    
    // get CryptoPhoto authentication session
    if (($sid = $req->get('sid', null)) != null) {
      // initialize CryptoPhoto storage
      $cps = new \Drupal\cryptophoto\Storage\CryptoPhotoStorage;

      // get authentication details
      $ad = $cps->getAuth($sid);

      // get user ID to be authenticated
      if ($ad) {
        $uid = $ad->uid;
      }
    }

    // test if we have a valid user
    if ($uid) {
      // see if the user POSTed something
      if ($req->isMethod('POST')) {
        // check for necessary fields
        if ($req->get('cp_auth') || $req->get('cp_phc')) {
          if (($req->get('token_response_field_row') && $req->get('token_response_field_col')) || $req->get('cp_phc')) {
            if ($req->get('token_selector') && ( ($req->get('token_response_field_row') && $req->get('token_response_field_col')) || $req->get('cp_phc')) ) {
              $selector = $req->get('token_selector');
              $row_code = $req->get('token_response_field_row', '');
              $col_code = $req->get('token_response_field_col', '');
              $cp_phc   = $req->get('cp_phc', '');
              $ip       = $_SERVER['REMOTE_ADDR'];

              // initialize CryptoPhoto
              $cp = new \CryptoPhotoUtils($server, $privatekey, $publickey, md5($salt . $uid));

              // verify codes
              $cp->verify_response($selector, $row_code, $col_code, $cp_phc, $ip);
  
              if ($cp->is_valid) {
                // codes ok. Delete the authentication session
                $cps->delAuth($sid);
                // log user in
                return $this->logUser($uid);
              } else {
                // authentication failed. Find out why
                $cperror = $cp->error;
                $error_message = t('Authentication error');

                if ($cperror == "incorrect-retry-count") {
                    $error_message = t('Invalid retry count');
                }

                if ($cperror == "incorrect-token-response") {
                    $error_message = t("Codes don't match");
                }
              }
            }
          } else {
            // empty input codes
            $error_message = t('You must fill in both codes');
          }
        }
      }

      // at this point we might have an error or no POST data
      if ($privatekey && $publickey && $salt) {
        // initialize CryptoPhoto if needed
        if (!$cp) {
          $cp = new \CryptoPhotoUtils($server, $privatekey, $publickey, md5($salt . $uid));
        }
        
        // attempt to start a new session
        $session = $cp->start_session($_SERVER['REMOTE_ADDR'], true);

        // test if new session started successfully
        if ($session[0] == TRUE) {
          // no point to continue if the user doesn't has a token
          if (!$cp->has_token)
            return $this->logUser($uid);

          // build authentication form
          $output['cp_auth'] = array(
            '#markup' => "<form method='POST'><div id='cp_widget' class='authform'></div><div class='authform'><input type='submit' name='cp_auth' class='button' value='Authenticate'></div></form>",
            '#allowed_tags' => ['form', 'div', 'input']
          );
          $output['#attached']['library'][] = 'cryptophoto/add_script';
          $output['#attached']['library'][] = 'cryptophoto/auth_form_css';
          $output['#attached']['drupalSettings']['cryptophoto']['as_url'] = $cp->get_auth_widget_url();
        } else {
          // something went wrong, look for the reason why
          if ($session[1] == "incorrect-timestamp") {
              $error_message = t('Incorrect timestamp');
          } elseif (strpos($session[1],"ip-not-allowed") !== false) {
              $error_message = t('IP not allowed');
          } else if ($session[1] == "incorrect-signature" || $session[1] == "incorrect-api-request") {
              $error_message = t('Invalid signature or request');
          } else $error_message = t('Internal error');
        }
      } else {
        $error_message = t('Not configured');
      }
    } else {
      $error_message = t('Missing session identifier');
    }

    // format error messages if any
    if ($error_message) {
      $output['error_message'] = array(
        '#markup' => "<div class='messages messages--error'>". $error_message . "</div>"
      );
    }

    return $output;
  }
}

?>
